#!/usr/bin/env perl
# Author test: CPANTS kwalitee hygiene (has_changelog, has_readme,
# use_strict, has_abstract_in_pod, no_symlinks, license-in-source, ...).
#
# Gated behind RELEASE_TESTING / AUTHOR_TESTING, as kwalitee checks are
# release-time concerns. META.yml / META.json are excluded: they are
# generated by `make dist`, so they are absent in a plain git checkout
# (the toolchain validates them on the built tarball).
use strict;
use warnings;
use Test::More;

plan skip_all => 'set RELEASE_TESTING or AUTHOR_TESTING to run kwalitee checks'
    unless $ENV{RELEASE_TESTING} || $ENV{AUTHOR_TESTING};

eval { require Test::Kwalitee; Test::Kwalitee->import('kwalitee_ok'); 1 }
    or plan skip_all => 'Test::Kwalitee required for kwalitee checks';

kwalitee_ok(qw( -has_meta_yml -has_meta_json ));
done_testing;
